#include "header.h"
#include "flow.h"
#include "vibrato.h"

#include "breath.h"
#include "swap.h"
#include "biqud.h"
#include "ReedOsc.h"
#include "ReedClip.h"

#include "Bflow15.h"

$header(44100'4)

instr 1; instr 1 acts like "the player" and instr 2 like "the instrument"
;instr 2 must be active troughout the entire piece
;instr 1 is activated at each note event

;SCO SETUP
ifrequency = p4/4
print ifrequency; displays current note being played
ifreq = p4/4;cpspch(p4)
iamp = .5*p5/32768;1;ampdb(p5)

$flow(3)
$vibrato(100'6.5'3)
$breath(.0015'1000'3)

$swap(ittime'0'4)
endin


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
instr 2
;MIDIfied clarinet model
;variable embochure model for Waveguide Woodwinds
;loosely based on a JNMR article (Vol.24 pp.148-163)
;written by Gijs de Bruin & Maarten van Walstijm
;coded to Csound by Josep M Comajuncosas / dec98-feb99
;thanks to Marteen for his valuable help
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

ipi =  3.14159265359
;ixtiny = 0.000000004;small value
inumb_iter = 3; number of iterations, suggested 1 to 3
; used in the Newton-Rhapson method.

;default clarinet values
ifr = 2500; reed resonance frequency
iwr = 2*ipi*ifr; normalized in radians/cycle
ier = 44.4; = B^(-3/2) where B = airflow constant ~ 0.08
ihr = 0.0004;aperture of the reed in equilibrium
iir = 731;inertance of the air above the reed
iq  = 1.5;set q=2 for lip & double reed????
ist = 0.000177	
isr = 0.000146
iqr = 5.0
igr = iwr/iqr
iur = 0.0231

ic  = 343;sound velocity
irho = 1.21;air density
iz0 = (irho*ic)/ist
iv=1;iv = -1 (lip) or 1 (cane). Here "cane"
apm1 init 0;mouthpiece pressure at t=0 is 0
apr2 init 0;reflected pressure wave from the tube
kuf1 init 0;normal volume flow
kpr2 downsamp apr2;necessary to compute a k-rate variable

kp0 zkr 3;get flow value from zk location 3
adp1 = kp0 - apm1;pressure difference across reed

$ReedOsc(adp1'iv'ifr'iqr'iur'ihr)
$ReedClip(.00005'.4'ihr)
$Bflow15(iz0'iir'ier'inumb_iter)	

;CILYNDRICAL WAVEGUIDE (lowpass inverting)
;bore logic : swap if a new note has started
;this feature allows playing legato notes

kbore1 zkr 1
kbore2 zkr 2
;this conditional is normally skipped except at init time
;when bore1 can have its lenght undefined
kbore1 = (kbore1==0?kbore2:kbore1)
kweight zkr 4

;kdlt1 = -10/sr+1/kbore1; compensate for the Ur term
;kdlt2 = -10/sr+1/kbore2
kdlt1 = 1/kbore1
kdlt2 = 1/kbore2

;compute the phase delay
;caused by the open hole reflectance filter
;this delay will be substracted from the total waveguide length
;notice that the table storing the phase delay
;only works for frequencies up to 2kHz !
kphd1 tablei kbore1/2000,3,1
kphd2 tablei kbore2/2000,3,1

;there is an extra delay presumably caused by the reed motion (?)
;its empirically found value is added to the previous delay 
kphd1 = kphd1+1.8
kphd2 = kphd2+1.8


;calculations for fractional delay interpolation filter
;(by now a simple linear interpolator)
kdltn1 = int(-kphd1+(sr*kdlt1)/2)
kdltn2 = int(-kphd2+(sr*kdlt2)/2)
kdltf1 = frac(-kphd1+(sr*kdlt1)/2);note than time=int(time)+frac(time) ;-)
kdltf2 = frac(-kphd2+(sr*kdlt2)/2)

;pressure sent to two bores in parallel
bore1:
atemp11 delayr 1/20
api31 deltapn kdltn1
;open hole reflectance filter designed by Maarten (coeff. computed with MATLAB)
$biquad(api31'aprf1'-0.06734822596636'1'1.60866845795955'0.79033697044981'1'-0.00843592161383'-0.75860644369670)
;fractional delay filter , a 1st order Lagrange interpolator < --- IMPROVE THIS!!!!!!!
$biquad(aprf1'afdf12'1'1-kdltf1'kdltf1'0'1'0'0)
delayw api2

bore2:
atemp21 delayr 1/20
api32 deltapn kdltn2
$biquad(api32'aprf2'-0.06734822596636'1'1.60866845795955'0.79033697044981'1'-0.00843592161383'-0.75860644369670)
$biquad(aprf2'afdf22'1'1-kdltf2'kdltf2'0'1'0'0)
delayw api2

;signal reaching the bore closed end
apr3 = kweight*afdf12 + (1-kweight)*afdf22

;output signal
api3 = kweight*api31 + (1-kweight)*api32

;SOUND OUTPUT

;High-Pass Filter: 
$biquad(api3'aout'-0.834797947580800'1'-1.996950273367395'0.996978312332094'1'-1.706660482696107'0.707131817477271)

out aout*100;crude scaling

;update variables for next pass
kuf1 = kuf2
apm1 = apm2
apr2 = apr3


zkcl 3,3;clean mixer
; (flow pressure contributed by the 2 overlapping notes)
endin
